from ohm2_handlers_light.decorators import ohm2_handlers_light_safe_request

def ohm2_countries_light_safe_request(function):
	return ohm2_handlers_light_safe_request(function, "ohm2_countries_light")



from django.contrib import admin
from . import models as ohm2_countries_light_models

"""
class ModelAdmin(admin.ModelAdmin):
	pass

admin.site.register(ohm2_countries_light_models.Model, ModelAdmin)
"""


# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-08-01 04:07
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.utils.timezone
import ohm2_countries_light.managers


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identity', models.CharField(max_length=2048, unique=True)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
                ('last_update', models.DateTimeField(default=django.utils.timezone.now)),
                ('name', models.CharField(max_length=512, validators=[django.core.validators.MinLengthValidator(1)])),
                ('official_name', models.CharField(max_length=512, validators=[django.core.validators.MinLengthValidator(1)])),
                ('alpha_2', models.CharField(max_length=64, validators=[django.core.validators.MinLengthValidator(2)])),
                ('alpha_3', models.CharField(max_length=64, validators=[django.core.validators.MinLengthValidator(3)])),
                ('numeric', models.IntegerField()),
                ('flag_small', models.ImageField(upload_to=ohm2_countries_light.managers.country_flag_small_upload_to)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from django.core.exceptions import ObjectDoesNotExist
from django.contrib.auth.models import User
from django.utils import timezone
from django.utils.translation import ugettext as _
from django.db.models import Q
from ohm2_handlers_light import utils as h_utils
from ohm2_handlers_light.definitions import RunException
from . import models as ohm2_countries_light_models
from . import errors as ohm2_countries_light_errors
from . import settings
import os, time, random


random_string = "Re4Wwe69YRenGntPO8ADErMBs1Y4hCLQ"

	

def create_country(name, official_name, alpha_2, alpha_3, numeric, **kwargs):
	kwargs["name"] = name.strip().title()
	kwargs["official_name"] = official_name.strip().title()
	kwargs["alpha_2"] = alpha_2.strip()
	kwargs["alpha_3"] = alpha_3.strip()
	kwargs["numeric"] = numeric

	flag_small = kwargs.get("flag_small")
	if flag_small:
		kwargs["flag_small"] = process_country_flag_small_image(flag_small)
		
	return h_utils.db_create(ohm2_countries_light_models.Country, **kwargs)

def process_country_flag_small_image(image, **kwargs):
	extension = h_utils.get_file_extension(image.name)
	if extension is None:
		return image
	else:
		extension = extension.lower()

	
	save_options = {
		"max_width": 48,
		"max_height": 48,
	}
		
	if extension in ["jpg", "jpeg"]:
		save_options["format"] = "JPEG"
		save_options["quality"] = 95
		save_options["optimize"] = True

	elif extension == "png":
		save_options["format"] = "PNG"
		save_options["quality"] = 95
		save_options["optimize"] = True
	
	else:
		pass


	options = {
		"save_options": save_options,
	}
	return h_utils.process_uploaded_image_2(image, **options)

def get_country(**kwargs):
	return h_utils.db_get(ohm2_countries_light_models.Country, **kwargs)

def get_or_none_country(**kwargs):
	return h_utils.db_get_or_none(ohm2_countries_light_models.Country, **kwargs)

def filter_country(**kwargs):
	return h_utils.db_filter(ohm2_countries_light_models.Country, **kwargs)		

def update_country(country, **kwargs):
	params = {}

	flag_small = kwargs.get("flag_small")
	if flag_small:
		params["flag_small"] = process_country_flag_small_image(flag_small)	

	return h_utils.db_update(country, **params)




from django.core.management.base import BaseCommand, CommandError
from ohm2_handlers_light import utils as h_utils
from ohm2_countries_light import utils as ohm2_countries_light_utils
from ohm2_countries_light import settings
from collections import defaultdict
import os, pycountry

class Command(BaseCommand):
	
	def add_arguments(self, parser):
		pass #parser.add_argument('-f', '--foo')

	def handle(self, *args, **options):
		# foo = options["foo"]
		
		"""
		files = defaultdict(dict)
		sizes = ["16", "24", "32", "48"]

		
		for c in pycountry.countries:

			alpha_2 = c.alpha_2.lower()
			name = c.name
			
			files[name] = defaultdict(dict)
			for size in sizes:
				files[name][size] = True


			for size in sizes:
				size_dst_path = os.path.join(settings.FLAGS_BASE_PATH, size, alpha_2 + ".png")	

				files[name][size] &= os.path.isfile(size_dst_path)

		
		
		all_around = []
		not_around = []
		for country, sizes in files.items():
			all_exist = True
			for v in sizes.values():
				all_exist &= v
			

			print(country, sizes)	



			if all_exist:
				all_around.append(country)
			else:
				not_around.append(country)

		print(sorted(all_around))
		"""


		"""
		
		files = defaultdict(dict)
		
		
		for c in pycountry.countries:

			alpha_2 = c.alpha_2.lower()
			name = c.name
			
			files[name] = defaultdict(bool)

			dst_path = os.path.join("/Users/tonra/Downloads/flags-normal", alpha_2 + ".png")
			files[name] = os.path.isfile(dst_path)
			print(dst_path)
		
		
		all_around = []
		not_around = []
		for country, exist in files.items():
			if exist:
				all_around.append(country)
			else:
				not_around.append(country)

		print(sorted(not_around), len(not_around))

		"""


		"""
		files = defaultdict(dict)
		
		
		for c in pycountry.countries:

			alpha_2 = c.alpha_2.lower()
			name = c.name
			
			files[name] = defaultdict(bool)

			dst_path = os.path.join(settings.FLAGS_BASE_PATH, alpha_2 + ".png")
			files[name] = os.path.isfile(dst_path)
		
		
		all_around = []
		not_around = []
		for country, exist in files.items():
			if exist:
				all_around.append(country)
			else:
				not_around.append(country)

		print(sorted(not_around), len(not_around))
		"""

		"""
		from django.core.files import File
		from django.core.files.temp import NamedTemporaryFile
		from PIL import Image
		

		dst_path = os.path.join(settings.FLAGS_SMALL_BASE_PATH, "cl.png")

		local = NamedTemporaryFile(delete = True, suffix = ".png")
		with open(dst_path, "rb") as f:
			
			keep_reading = True
			while keep_reading:
				content = f.read(32)
				if content == b"":
					keep_reading = False
				else:
					local.write( content )

		

		local.flush()
		local.seek(0)
		django_file = File(local)


		print(django_file)


		im = Image.open(django_file.name)

		f2 = NamedTemporaryFile(suffix = ".png", delete = False)
		im.save(f2.name)

		
		local = NamedTemporaryFile(delete = True, suffix = ".png")
		with open(f2.name, "rb") as f:
			
			keep_reading = True
			while keep_reading:
				content = f.read(32)
				if content == b"":
					keep_reading = False
				else:
					local.write( content )

		

		local.flush()
		local.seek(0)
		process_image = File(local)


		country = ohm2_countries_light_utils.filter_country()[0]
		h_utils.db_update(country, flag_small = process_image)
		"""



		"""
		from django.core.files import File
		from django.core.files.temp import NamedTemporaryFile
		from PIL import Image
		

		dst_path = os.path.join(settings.FLAGS_SMALL_BASE_PATH, "cl.png")

		django_file = open(dst_path, "rb")


		print(django_file)


		im = Image.open(django_file.name)

		f2 = NamedTemporaryFile(suffix = ".png", delete = False)
		im.save(f2.name)

		
		process_image = h_utils.get_local_image_2(f2.name)


		country = ohm2_countries_light_utils.filter_country()[0]
		h_utils.db_update(country, flag_small = process_image)
		"""


		dst_path = os.path.join(settings.FLAGS_SMALL_BASE_PATH, "cl.png")

		uploaded_image = h_utils.new_local_file(dst_path)


		processed_image = h_utils.process_uploaded_image_2(uploaded_image)



from django.core.management.base import BaseCommand, CommandError
from django.core.files import File
from ohm2_handlers_light import utils as h_utils
from ohm2_countries_light import utils as ohm2_countries_light_utils
from ohm2_countries_light import settings
import os, pycountry

class Command(BaseCommand):
	
	def add_arguments(self, parser):
		pass #parser.add_argument('-f', '--foo')

	def handle(self, *args, **options):
		# foo = options["foo"]
		
		
		for c in pycountry.countries:

			
			flag_small_dst_path = os.path.join(settings.FLAGS_SMALL_BASE_PATH, c.alpha_2.lower() + ".png")
			flag_small_exist = os.path.isfile(flag_small_dst_path)
			if flag_small_exist:
				flag_small = h_utils.new_local_file(flag_small_dst_path)
			else:
				flag_small = None	

			
			official_name = getattr(c, "official_name", c.name)
			country = ohm2_countries_light_utils.get_or_none_country(alpha_3 = c.alpha_3)
			if country is None:
				country = ohm2_countries_light_utils.create_country(c.name, official_name, c.alpha_2, c.alpha_3, c.numeric,
													  				flag_small = flag_small)
			else:
				country = ohm2_countries_light_utils.update_country(country, flag_small = flag_small)
			
			
				


		
			


			


from rest_framework import serializers
from . import models as ohm2_countries_light_models
from . import settings



class Country(serializers.ModelSerializer):
	class Meta:
		model = ohm2_countries_light_models.Country
		fields = (
			'identity',
			'created',
			'last_update',
			'name',
			'official_name',
			'alpha_2',
			'alpha_3',
			'numeric',
			'flag_small_url',
		)



from django.db import models
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.core.validators import MinLengthValidator, MaxLengthValidator
from django.utils import timezone
from django.utils.translation import ugettext as _
from ohm2_handlers_light.models import BaseModel
from . import managers
from . import settings



class Country(BaseModel):
	name = models.CharField(max_length = settings.STRING_NORMAL, validators = [MinLengthValidator(1)])
	official_name = models.CharField(max_length = settings.STRING_NORMAL, validators = [MinLengthValidator(1)])
	alpha_2 = models.CharField(max_length = settings.STRING_SHORT, validators = [MinLengthValidator(2)])
	alpha_3	= models.CharField(max_length = settings.STRING_SHORT, validators = [MinLengthValidator(3)])
	numeric = models.IntegerField()
	flag_small = models.ImageField(upload_to = managers.country_flag_small_upload_to)


	@property
	def flag_small_url(self):
		if self.flag_small:
			return self.flag_small.url
		return None	